import sys
import re
from xen_connection import xen_connection

class check_vm:
    exit_code = 0
    options = None
    cpure = None
    ifre = None
    diskre = None

    def __init__(self):
        self.cpure = re.compile('cpu\d+')
        self.ifre = re.compile('if(.+)')
        self.diskre = re.compile('disk(.+)')

    def set_options(self, options):
        if not options.uuid:
            raise Exception("Missing parameter uuid(-i)")

        if not options.sub:
            raise Exception("Missing parameter sub(-s)")

        if not options.warning:
            raise Exception("Missing parameter warning(-w)")

        if not options.critical:
            raise Exception("Missing parameter critical(-c)")

        self.options = options

    def create_xen_connection(self):
        for proto in ['http://','https://']:
            conn = xen_connection()
            conn.url = proto + self.options.host
            conn.username = self.options.user
            conn.password = self.options.password
            if (conn):
                return conn
        return

    def memory(self, stats):

        allocated = stats['memory'] / 1024 / 1024

        ecode = 0
        if (allocated > self.options.critical):
            ecode = 2
        elif (allocated > self.options.warning):
            ecode = 1

        sys.stdout.write("Memory allocated: %.1fMB |" % allocated)
        sys.stdout.write("allocated=%.1fMB;" % allocated)
        sys.stdout.write("%.1f;%.1f;;\n" % (self.options.warning, self.options.critical))

        return ecode

    def cpu(self, stats):
        if self.options.sub not in stats:
            raise Exception("Unable to locate %s" % self.options.sub)

        load = stats[self.options.sub]
        ecode = 0
        if (load > self.options.critical):
            ecode = 2
        elif (load > self.options.warning):
            ecode = 1

        sys.stdout.write("%s load: %.4f|" % (self.options.sub, load))
        sys.stdout.write("load=%.4f;" % load)
        sys.stdout.write("%.4f;%.4f;;\n" % (self.options.warning, self.options.critical))

        return ecode

    def disk(self, stats):

        match = self.diskre.search(self.options.sub)
        if not match:
            raise Exception("Unable to locate counter for disk " % self.options.sub)


        diskname = match.group(1)
        read_idx = "vbd_%s_read" % diskname
        write_idx = "vbd_%s_write" % diskname

        if read_idx not in stats:
            raise Exception("Unable to locate disk counter %s" % read_idx)
        if write_idx not in stats:
            raise Exception("Unable to locate disk counter %s" % write_idx)

        read = stats[read_idx]
        write = stats[write_idx]


        ecode = 0
        if read > self.options.critical or write > self.options.critical:
            ecode = 2
        elif read > self.options.warning or write > self.options.warning:
            ecode = 1

        sys.stdout.write("disk %s reads: %.2f writes: %.2f|" % (diskname, read, write))
        sys.stdout.write("reads=%.2f;%.2f;%.2f;; " %  (read, self.options.warning, self.options.critical))
        sys.stdout.write("writes=%.2f;%.2f;%.2f;; " %  (write, self.options.warning, self.options.critical))

        print
        return ecode

    def interface(self, stats):

        match = self.ifre.search(self.options.sub)
        if not match:
            raise Exception("Unable to locate counter for interface " % self.options.sub)

        ifname = match.group(1)
        tx_idx = "vif_%s_tx" % ifname
        rx_idx = "vif_%s_rx" % ifname
        txe_idx = "vif_%s_tx_errors" % ifname
        rxe_idx = "vif_%s_rx_errors" % ifname

        if tx_idx not in stats:
            raise Exception("Unable to locate interface counter %s" % tx_idx)
        if rx_idx not in stats:
            raise Exception("Unable to locate interface counter %s" % rx_idx)

        upload = stats[tx_idx] / 1024
        download = stats[rx_idx] / 1024

        ecode = 0
        if upload > self.options.critical or download > self.options.critical:
            ecode = 2
        elif upload > self.options.warning or download > self.options.warning:
            ecode = 1

        sys.stdout.write("virtual interface vif%s upload: %.2fkbps download: %.2fkbps|" % (ifname, upload, download))
        sys.stdout.write("upload=%.2fkbps;%.2f;%.2f;; " %  (upload, self.options.warning, self.options.critical))
        sys.stdout.write("download=%.2fkbps;%.2f;%.2f;; " %  (download, self.options.warning, self.options.critical))
        if rxe_idx in stats:
            sys.stdout.write("rx_errors=%s;;;; " %  stats[rxe_idx])
        if txe_idx in stats:
            sys.stdout.write("tx_errors=%s;;;; " %  stats[txe_idx])

        print
        return ecode

    def run(self):

        conn = self.create_xen_connection()
        stats = conn.get_vm_stats(self.options.uuid)

        if self.options.sub == 'memory':
            ecode = self.memory(stats)
        elif self.cpure.match(self.options.sub):
            ecode = self.cpu(stats)
        elif self.ifre.match(self.options.sub):
            ecode = self.interface(stats)
        elif self.diskre.match(self.options.sub):
            ecode = self.disk(stats)
        else:
            raise Exception("Invalid sub option(-s)")

        return ecode
