<?php

use Models\WebServices\Mapper\Mapper as Mapper;

class XenManager
{
    protected $mapper;

    public function __construct(Mapper $mapper = null)
    {
        $this->mapper = $mapper ?: new Mapper();
    }

    private $protocol = array("http","https");
    public $instpath = "/usr/local/opmon/share/modelgroups/xenserver/manager";

    function run_command($cmd, $data)
    {
        foreach ($this->protocol as $proto) {
            $credentials = sprintf(
                " -u %s -p %s -h %s",
                escapeshellarg($data->globalusername),
                escapeshellarg($data->globalpassword),
                escapeshellarg($proto."://".$data->globaladdress)
            );

            $runner = new ShellCommandRunner();
            $runner->run($cmd.$credentials);
            $output = json_decode($runner->get_stdout());
            if ($this->validate_return($output) === true)
                return $output;
        }
        return false;
    }

    function validate_return($output)
    {
        if ($output === NULL || $output === "null")
            return false;

        if (!isset($output->success))
            return false;

        if ($output->success === false)
            return false;

        return true;
    }

    function get_host_cpu_counters($data) {

        if (!isset($data->globalusername) || empty($data->globalusername))
            throw new Exception("Please provide an globalusername");

        if (!isset($data->globalpassword) || empty($data->globalpassword))
            throw new Exception("Please provide a globalpassword");

        if (!isset($data->globaladdress) || empty($data->globaladdress))
            throw new Exception("Please provide a XenAPI globaladdress");

        $cmd = sprintf("%s/scripts/xenaccess.py -c",
            $this->instpath
        );

        $ret = $this->run_command($cmd, $data);

        if (!isset($ret->data))
            throw new Exception("Unable to retrieve host counters");

        $json = array();
        $seen = array();
        foreach($ret->data as $counter) {
            if (preg_match("/^cpu(\d+)$/", $counter, $matches)) {

                $name = "CPU ".$matches[1];
                if (in_array($name, $seen))
                    continue;

                $aux = array(
                    "key" => $name,
                    "value" => "cpu".$matches[1]
                );

                array_push($seen, $name);
                array_push($json, $aux);

                $this->mapper->addElement(
                    $name,
                    $name,
                    array("CPU-cpu" => "cpu".$matches[1])
                );
                continue;
            }
        }
        if (empty($json)) {
            throw new Exception("There was no return from XenApi");
        }
        $this->mapper->setComboData($json);
        return $this->mapper->getOutput();
    }


    function get_host_memory_counters($data) {

        if (!isset($data->globalusername) || empty($data->globalusername))
            throw new Exception("Please provide an globalusername");

        if (!isset($data->globalpassword) || empty($data->globalpassword))
            throw new Exception("Please provide a globalpassword");

        if (!isset($data->globaladdress) || empty($data->globaladdress))
            throw new Exception("Please provide a XenAPI globaladdress");

        $cmd = sprintf("%s/scripts/xenaccess.py -c",
            $this->instpath
        );

        $ret = $this->run_command($cmd, $data);

        if (!isset($ret->data))
            throw new Exception("Unable to retrieve host counters");

        $json = array();
        $seen = array();
        foreach($ret->data as $counter) {

            if (preg_match("/.*memory.*/", $counter) && !in_array("memory", $seen)) {
                $aux = array(
                    "key" => "Memory",
                    "value" => "memory"
                );

                array_push($seen, "memory");
                array_push($json, $aux);
                $this->mapper->addElement(
                    "Memory ",
                    "Memory ",
                    array("Memory-memory" => "memory")
                );
                continue;
            }
        }
        if (empty($json)) {
            throw new Exception("There was no return from XenApi");
        }
        $this->mapper->setComboData($json);
        return $this->mapper->getOutput();
    }

    function get_host_interface_counters($data) {

        if (!isset($data->globalusername) || empty($data->globalusername))
            throw new Exception("Please provide an globalusername");

        if (!isset($data->globalpassword) || empty($data->globalpassword))
            throw new Exception("Please provide a globalpassword");

        if (!isset($data->globaladdress) || empty($data->globaladdress))
            throw new Exception("Please provide a XenAPI globaladdress");

        $cmd = sprintf("%s/scripts/xenaccess.py -c",
            $this->instpath
        );

        $ret = $this->run_command($cmd, $data);

        if (!isset($ret->data))
            throw new Exception("Unable to retrieve host counters");

        $json = array();
        $seen = array();
        foreach($ret->data as $counter) {
            if (preg_match("/^pif_([^_]+)_.+$/", $counter, $matches)) {
                $name = "Interface ".$matches[1];
                if (in_array($name, $seen))
                    continue;

                $aux = array(
                    "key" => $name,
                    "value" => "if".$matches[1]
                );

                array_push($seen, $name);
                array_push($json, $aux);
                $this->mapper->addElement(
                    "Interface ".$matches[1],
                    $name,
                    array("Interface_interface" => "if".$matches[1])
                );
                continue;
            }
        }
        if (empty($json)) {
            throw new Exception("There was no return from XenApi");
        }
        $this->mapper->setComboData($json);
        return $this->mapper->getOutput();
    }
}
?>

