#!/usr/bin/env python
import sys
sys.path.append('/usr/local/opmon/libexec/modelgroups/xenserver/agents')
import getopt
import classes
import simplejson


def print_error(str):
	e = { "success": False, "message": str }
	print simplejson.dumps(e)


def print_ok(obj):
	o = { "success": True, "data": obj }
	print simplejson.dumps(o)

def new_xenconn(params):
	c = classes.xen_connection()
	c.url = params['url']
	c.username = params['user']
	c.password = params['password']
	return c

def list_host_counters(params):
	c = new_xenconn(params)
	try:
		data = c.get_host_stats()
	except:
		print_error("Unexpected error")
		return 3

	counters = []
	for c in data:
		counters.append(c)
	
	print_ok(counters)
	return 0


def list_vms(params):
	c = new_xenconn(params)
	try:
		data = c.get_vms()
	except:
		print_error("Unexpected error")
		return 3

	vms = []
	for vm in data:
		auxvm = {}
		auxvm['value'] = vm['uuid']
		auxvm['key'] = vm['name_label']
		vms.append(auxvm)

	print_ok(vms)
	return 0

def list_vm_stats(params):
	if params['uuid'] == None:
		print_error("Missing uuid(-i)")
		return 3

	c = new_xenconn(params)
	print_ok(c.get_vm_stats(params['uuid']))
	return 0

def validate_params(params):
	if params['user'] == None:
		print_error("Missing user(-u)")
		return False

	if params['password'] == None:
		print_error("Missing password(-p)")
		return False

	if params['url'] == None:
		print_error("Missing url(-h)")
		return False
	
	return True


if __name__ == "__main__":

	params = {}
	params['user'] = None
	params['password'] = None
	params['url'] = None
	params['uuid'] = None
	params['list_vms'] = False
	params['list_host_counters'] = False

	try:
		opts, args = getopt.getopt(sys.argv[1:], "clh:u:p:i:")
	except:
		print_error("Invalid command line arguments")
		sys.exit(3)
	
	for o, a in opts:
		if o == '-l':
			params['list_vms'] = True
		if o == '-c':
			params['list_host_counters'] = True
		elif o == '-u':
			params['user'] = a
		elif o == '-h':
			params['url'] = a
		elif o == '-p':
			params['password'] = a
		elif o == '-i':
			params['uuid'] = a

	if validate_params(params) == False:
		sys.exit(3)

	if params['list_vms'] == True:
		sys.exit(list_vms(params))
	
	if params['list_host_counters'] == True:
		sys.exit(list_host_counters(params))

	sys.exit(list_vm_stats(params))
